#ifndef ZLIB_UNZIP
#define ZLIB_UNZIP

[Files]
Source: "{#emit ExtractFilePath(__PATHFILENAME__)}\zlibwapi.dll"; DestDir: "{tmp}"; Flags: dontcopy

[Code]
type
	unzFile = Integer;

	Ttm_unz = record
		tm_sec,            (* seconds after the minute - [0,59] *)
		tm_min,            (* minutes after the hour - [0,59] *)
		tm_hour,           (* hours since midnight - [0,23] *)
		tm_mday,           (* day of the month - [1,31] *)
		tm_mon,            (* months since January - [0,11] *)
		tm_year: Integer;  (* years - [1980..2044] *)
	end;
	
	Tunz_file_info = record
		version,              (* version made by                 2 bytes *)
		version_needed,       (* version needed to extract       2 bytes *)
		flag,                 (* general purpose bit flag        2 bytes *)
		compression_method,   (* compression method              2 bytes *)
		dosDate,              (* last mod file date in Dos fmt   4 bytes *)
		crc,                  (* crc-32                          4 bytes *)
		compressed_size,      (* compressed size                 4 bytes *)
		uncompressed_size,    (* uncompressed size               4 bytes *)
		size_filename,        (* filename length                 2 bytes *)
		size_file_extra,      (* extra field length              2 bytes *)
		size_file_comment,    (* file comment length             2 bytes *)

		disk_num_start,       (* disk number start               2 bytes *)
		internal_fa,          (* internal file attributes        2 bytes *)
		external_fa: Integer; (* external file attributes        4 bytes *)

		tmu_date: Ttm_unz;
	end;

function unzOpen(path: PAnsiChar): unzFile;
  external 'unzOpen@files:zlibwapi.dll stdcall';
function unzClose(file: unzFile): Integer; 
  external 'unzClose@files:zlibwapi.dll stdcall';
function unzGoToFirstFile(file: unzFile): Integer;
  external 'unzGoToFirstFile@files:zlibwapi.dll stdcall';
function unzGoToNextFile(file: unzFile): Integer;
  external 'unzGoToNextFile@files:zlibwapi.dll stdcall';
function unzLocateFile(file: unzFile; szFileName: PAnsiChar; iCaseSensitivity: Integer): Integer;
  external 'unzLocateFile@files:zlibwapi.dll stdcall';
function unzOpenCurrentFile(file: unzFile): Integer;
  external 'unzOpenCurrentFile@files:zlibwapi.dll stdcall';
function unzCloseCurrentFile(file: unzFile): Integer;
  external 'unzCloseCurrentFile@files:zlibwapi.dll stdcall';
function unzReadCurrentFile(file: unzFile; buf: PAnsiChar; len: Integer): Integer;
  external 'unzReadCurrentFile@files:zlibwapi.dll stdcall';
function unzGetCurrentFileInfo(file: unzFile; var pfile_info: Tunz_file_info; szFileName: PAnsiChar; fileNameBufferSize: Integer; extraField, extraFieldBufferSize, szComment, commentBufferSize: Integer): Integer;
  external 'unzGetCurrentFileInfo@files:zlibwapi.dll stdcall';

Const
  // zlib/miniunzip constants
	UNZ_OK = 0;
	UNZ_END_OF_LIST_OF_FILE = -100;
	Z_ERRNO = -1;
	UNZ_ERRNO = Z_ERRNO;
	UNZ_EOF = 0;
	UNZ_PARAMERROR = -102;
	UNZ_BADZIPFILE = -103;
	UNZ_INTERNALERROR = -104;
	UNZ_CRCERROR = -105;

  // Buffer sizes
	BUFFER_SIZE = 16384;
	NAME_BUFFER = 1024;

function TranslateZipError(ErrorConstant: Integer): String;
begin
  case ErrorConstant of
    UNZ_END_OF_LIST_OF_FILE: Result := 'end of file list reached';
    UNZ_EOF: Result := 'end of file reached';
    UNZ_PARAMERROR: Result := 'invalid parameter given';
    UNZ_BADZIPFILE: Result := 'bad zip file';
    UNZ_INTERNALERROR: Result := 'internal error';
    UNZ_CRCERROR: Result := 'CRC error: corrupt zip file';
    UNZ_ERRNO: Result := SysErrorMessage(DLLGetLastError());
    else Result := Format('Unknown error: %.8x', [ErrorConstant]);
  end;
end;

function ExtractFile(ZipFileID: unzFile; FileInfo: Tunz_file_info; FileName: String): Boolean;
var
  RetVal: Integer;
  Data, OutData: AnsiString;
begin
	try
    RetVal := 1;
    Data := StringOfChar(#0, BUFFER_SIZE);

		while (RetVal > 0) do
		begin
			RetVal := unzReadCurrentFile(ZipFileID, Data, BUFFER_SIZE);
			if RetVal < 0 then
        RaiseException(TranslateZipError(RetVal) + ', File: ' + FileName);

			if FileInfo.uncompressed_size > BUFFER_SIZE then
        OutData := OutData + Copy(Data, 0, BUFFER_SIZE)
			else if FileInfo.uncompressed_size > 0 then
		    OutData := OutData + Copy(Data, 0, FileInfo.uncompressed_size);

			FileInfo.uncompressed_size := FileInfo.uncompressed_size - BUFFER_SIZE;
		end;

		Result := (RetVal = 0) and SaveStringToFile(FileName, OutData, False);
	except
		Log('UnZip Error: ' + GetExceptionMessage());
		Result := False;
  end;
end;

function UnzipFile(const ZipFile: String; FileName: String): Boolean;
var
  ZipFileID: unzFile;
	FileInfo: Tunz_file_info;
	NameInZip: AnsiString;
	RetVal: Integer;
begin
	Result := False;

	ZipFileID := unzOpen(ZipFile);

	if ZipFileID = 0 then
		exit;

  RetVal := unzLocateFile(ZipFileID, FileName, 2);
	if RetVal <> UNZ_OK then begin
		Log('UnZip Error: ' + AddPeriod(TranslateZipError(RetVal)));
		unzClose(ZipFileID);
		exit;
	end;

  RetVal := unzOpenCurrentFile(ZipFileID);
	if RetVal <> UNZ_OK then begin
		Log('UnZip Error: ' + AddPeriod(TranslateZipError(RetVal)));
		unzClose(ZipFileID);
		exit;
	end;

  NameInZip := StringOfChar(#0, NAME_BUFFER);
	unzGetCurrentFileInfo(ZipFileID, FileInfo, NameInZip, NAME_BUFFER, 0, 0, 0, 0);

  if Length(FileName) = 0 then
    FileName := NameInZip;

  Result := ExtractFile(ZipFileID, FileInfo, FileName);

  RetVal := unzCloseCurrentFile(ZipFileID);
	if RetVal <> UNZ_OK then begin
		Log('UnZip Error: ' + AddPeriod(TranslateZipError(RetVal)));
		Result := False;
	end;

	unzClose(ZipFileID);
end;

function UnzipAllFiles(const ZipFile: String; DirPath: String): Boolean;
var
  ZipFileID: unzFile;
	FileInfo: Tunz_file_info;
	FileName: AnsiString;
  RetVal: Integer;
begin
	Result := False;

	ZipFileID := unzOpen(ZipFile);

	if ZipFileID = 0 then
		exit;

  RetVal := unzGoToFirstFile(ZipFileID);
  if RetVal <> UNZ_OK then begin
		Log('UnZip Error: ' + AddPeriod(TranslateZipError(RetVal)));
		unzClose(ZipFileID);
		exit;
	end;

  repeat begin
    Result := False;
    RetVal := unzOpenCurrentFile(ZipFileID);
    if RetVal <> UNZ_OK then begin
      Log('UnZip Error: ' + AddPeriod(TranslateZipError(RetVal)));
		  unzClose(ZipFileID);
      exit;
	  end;

    FileName := StringOfChar(#0, NAME_BUFFER);
	  unzGetCurrentFileInfo(ZipFileID, FileInfo, FileName, NAME_BUFFER, 0, 0, 0, 0);

	  Result := ExtractFile(ZipFileID, FileInfo, DirPath + FileName);

    RetVal := unzCloseCurrentFile(ZipFileID);
    if RetVal <> UNZ_OK then begin
      Result := False;
		  Log('UnZip Error: ' + AddPeriod(TranslateZipError(RetVal)));
      unzClose(ZipFileID);
      exit;
    end;
  end;
  until unzGoToNextFile(ZipFileID) <> UNZ_OK;

	unzClose(ZipFileID);
end;

#endif
