#include 'InnoTools\it_download.iss'
#include 'UnZip\unzip.iss'

#include 'utils.iss'

[code]
var
  DownloaderUpdateLoc, DownloaderVerFile: String;
  DownloaderSkipCheck, DownloaderSkipComponents: Boolean;

procedure ExitProcess(exitCode:integer);
  external 'ExitProcess@kernel32.dll stdcall';

procedure Downloader_Initialize();
begin
  // ITD init call (currently NOP, but including it anyways)
  ITD_Init();

  // Where updater files are saved
  DownloaderUpdateLoc := ExpandConstant('{tmp}\NewApexInstaller.exe');
  DownloaderVerFile := ExpandConstant('{tmp}\ApexInstallerVer.ini');

  // In case download fails permit user to continue installation
  ITD_SetOption('UI_AllowContinue', '1');
end;

procedure Downloader_OnUpdateCheckComplete(DownloadPage: TWizardPage);
var
  ErrorCode: Integer;
begin
  MsgBox(ITD_GetString(ITDS_Update_WillLaunch), mbInformation, MB_OK);

  ITD_AfterSuccess := nil;

  if ShellExec('open', DownloaderUpdateLoc, '/updated',
      ExtractFilePath(DownloaderUpdateLoc), SW_SHOW, ewNoWait, ErrorCode) then
      ExitProcess(1);  
end;

function Downloader_CheckForUpdates(PreviousPage: Integer): Integer;
var
  i: Integer;
  DownloaderUpdatePage: TOutputProgressWizardPage;
  ourVersion, theirVersion, installerLocation, updateMsg: String;
begin
  Result := PreviousPage;

  // We only perform this check once (don't retry if user hits back)
  if DownloaderSkipCheck then exit;

  //Are we being called by an updating setup? If so, don't ask to check for updates again!
  for i:= 1 to ParamCount do begin
    if UpperCase(ParamStr(i))= '/UPDATED' then exit;
  end;

  // Progress page for the update check
  DownloaderUpdatePage := CreateOutputProgressPage(
    ITD_GetString(ITDS_Update_Caption),
    ITD_GetString(ITDS_Update_Description)
  );

  try
    // We always check for updates, downloading one is optional
    WizardForm.Show();
    DownloaderUpdatePage.Show();
    DownloaderUpdatePage.SetText(ITD_GetString(ITDS_Update_Checking),'');
    DownloaderUpdatePage.SetProgress(2,10);

    GetVersionNumbersString(ExpandConstant('{srcexe}'), ourVersion);

    if ITD_DownloadFile('{#UpdateUrl}', DownloaderVerFile) = ITDERR_SUCCESS then begin
      theirVersion := GetIniString('VersionInfo', 'Version', '0.0', DownloaderVerFile);
      installerLocation := GetIniString('VersionInfo', 'DownloadUrl', '', DownloaderVerFile);
      DownloaderUpdatePage.SetProgress(9,10);

      // On silent installs, don't prompt for installer update
      if not WizardSilent() and (CompareVersions(Trim(theirVersion), Trim(ourVersion)) > 0) then begin
        // Version is newer
        updateMsg := ITD_GetString(ITDS_Update_NewAvailable);
        StringChangeEx(updateMsg, '%1', ourVersion, True);
        StringChangeEx(updateMsg, '%2', theirVersion, True);

        if MsgBox(updateMsg, mbConfirmation, MB_YESNO) = IDYES then begin
          ITD_AddFile(Trim(installerLocation), DownloaderUpdateLoc);
          ITD_AfterSuccess := @Downloader_OnUpdateCheckComplete;

          // Create the ITD GUI for downloading the updated installer
          Result := ITD_DownloadAfter(PreviousPage).ID;

          // Do not permit continuing installation after failure (ITDownload limitation)
          ITD_SetOption('UI_AllowContinue', '0');
        end;
      end;
    end;
    DownloaderUpdatePage.SetProgress(10,10);
  finally
    DownloaderUpdatePage.Hide();
    DownloaderSkipCheck := True;
  end;
end;

function Downloader_PrepareForComponents(PreviousPage: Integer): Integer;
begin
  Result := PreviousPage;
  ITD_ClearFiles();

  // We don't want to call this if user hits back button
  if not DownloaderSkipComponents then
    Result := ITD_DownloadAfter(PreviousPage).ID;

  DownloaderSkipComponents := True;
end;

procedure Downloader_AddComponent(ComponentName: String);
var
  FileUrl, FileLoc: String;
begin
  if IsComponentSelected(ComponentName) then begin
    FileLoc := ExpandConstant('{tmp}\' + ComponentName + '.zip');
    FileUrl := GetIniString('Component.' + ComponentName, 'DownloadUrl', '', DownloaderVerFile);
    if FileUrl <> '' then
      ITD_AddFile(FileUrl, FileLoc);
  end;
end;

procedure Downloader_InstallComponent(ComponentName, DestDir: String);
var
  FileLoc: String;
  DoExtract: Boolean;
begin
  FileLoc := ExpandConstant('{tmp}\' + ComponentName + '.zip');
  if IsComponentSelected(ComponentName) and FileExists(FileLoc) then begin
    DoExtract := GetIniString('Component.' + ComponentName, 'Extract', '1', DownloaderVerFile) <> '0';
    if DoExtract and UnzipAllFiles(FileLoc, DestDir) then
      Log('Installed remote component: ' + ComponentName + ' successfully.')
    else if not DoExtract then
      FileCopy(FileLoc, DestDir + ComponentName + '.zip', False);
    DeleteFile(FileLoc);
  end;
end;