#ifndef APEX_IS_UTILS
#define APEX_IS_UTILS

[Code]
// Check function for backwards compatibility
function IsCompatibilityMode(): Boolean;
begin
  Result := RegKeyExists(HKLM, '{#NSISKey}');
end;

// Check if user has FireFox installed (TODO: check up on smarter detection
function IsFirefoxInstalled(): Boolean;
begin
  Result := RegKeyExists(HKLM, 'SOFTWARE\Microsoft\Windows\CurrentVersion\App Paths\firefox.exe');
end;

// code:constant function for getting correct initial directory
function GetDefaultDir(Param: String): String;
begin
  if not IsCompatibilityMode() then
    Result := ExpandConstant('{pf}\{#Name}')
  else begin
    RegQueryStringValue(HKLM, '{#NSISKey}', 'UninstallString', Result);
    Result := ExtractFilePath(Result);
  end;
end;

// code:constant function for getting correct app executable path
function GetAppPath(Param: String): String;
begin
  if not Is64BitInstallMode() then
    Result := ExpandConstant('{app}\ApexDC.exe')
  else Result := ExpandConstant('{app}\ApexDC-x64.exe');
end;

// code:constant for getting the localised default types
function GetDefaultTypeName(msgName: String): String;
begin
  case msgName of
    'FullInstallation' : Result := SetupMessage(msgFullInstallation);
    'CompactInstallation' : Result := SetupMessage(msgCompactInstallation);
    'CustomInstallation' : Result := SetupMessage(msgCustomInstallation);
  else Result := CustomMessage(msgName);
  end;
end; 

{ Compares two version numbers (snagged from ITD example)
  Returns:
    this > that -> positive, 0 if this = that and if this < that -> negative. }
function CompareVersions(this, that: String): Integer;
var
  thisField, thatField: Integer;
begin
  while (Length(this) > 0) or (Length(that) > 0) do begin
    if (Pos('.', this) > 0) then begin
      // Read the first field from the string
      thisField := StrToIntDef(Copy(this, 1, Pos('.', this) - 1), 0);
      // Remove the first field from the string
      this := Copy(this, Pos('.', this) + 1, Length(this));
    end else begin
      thisField := StrToIntDef(this, 0);
      this := '';
    end;

    if (Pos('.', that) > 0) then begin
      // Read the first field from the string
      thatField := StrToIntDef(Copy(that, 1, Pos('.', that) - 1), 0);
      // Remove the first field from the string
      that := Copy(that, Pos('.', that) + 1, Length(that));
    end else begin
      thatField := StrToIntDef(that, 0);
      that := '';
    end;

    // Small optimization to the original
    Result := thisField - thatField;
    if Result <> 0 then exit;
  end;
end;

// Gets string width in pixels
function GetTextWidth(aText: String; aFont: TFont) : Integer;
var
  Bmp: TBitmap; 
begin
  Bmp := TBitmap.Create();
  try
    Bmp.Canvas.Font.Assign(aFont);
    Result := Bmp.Canvas.TextWidth(aText); 
  finally
    Bmp.Free;
  end;
end;

#endif
